# jit.R
#
# This makes sure that "is.ra", "jit", and "nojit" are defined.
#
# In the R environment, we define all these from scratch.
#
# In the Ra environment, "jit" is already defined internally.
# Also "nojit" is defined internally but needs some padding
# around it, which we add here.
#
# A note on the pos parameter for assign: originally we assigned
# to .GlobalEnv but with that we found that RCMD CHECK failed running the
# examples.  Hence the dirty trick of assigning to the base environment.
#
# Note also that we still get the following warnings from the RCMD CHECK:
#   "Functions in documentation object 'jit' but not in code: jit, nojit"
# We currently know of no way of getting rid of those warnings.

.onLoad <- function(lib, pkg) {
    pos = which(search() == "package:base")
    assign("is.ra", length(grep(" Ra ", R.version.string)) != 0, pos=pos)
    if (!is.ra) {
        # We are running standard R, so must define a dummy jit function.
        # We define jit's return value once and for all here,
        # so calls to jit are as efficient as possible.
        # Note that result is defined locally and is thus
        # invisible to the rest of the world.

        result <- c(0L, 0L, 0L)
        names(result) <- c("jit", "trace", "callers.jit")

        jit <- function(jit = NA, trace = 0) {
            if(is.na(jit))  # default arg?
                result      # show result
            else
                invisible(result)
        }
        assign("jit", jit, pos=pos)
    }
    nojit <- function(sym = NULL) {
        result <- NULL
        # use substitute else do_nojit gets the already evaluated sym
        if(is.ra)
            result <- .Internal(nojit(substitute(sym)))
        if(is.null(sym))        # default arg?
            result              # show nojit symbols
        else
            invisible(result)
    }
    assign("nojit", nojit, pos=pos)
}
